package javafxclasses;

import javafx.geometry.Point2D;
import javafx.geometry.Rectangle2D;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;

/**
 * A base class for geometric shapes.
 */
public abstract class ExampleShape {
    /**
     * The stroke paint for this shape.
     */
    public var strokePaint:Paint = Color.BLACK;

    /**
     * The fill paint for this shape, if it has one
     */
    public var fillPaint:Paint;

    /**
     * Returns whether this shape should be filled.
     * @return true to fill the shape.
     */
    public function shouldFill():Boolean {
        return fillPaint != null;
    }
    
    /**
     * Gets the area of the shape.
     * @return the area of the shape.
     */
    public abstract bound function getArea():Number;

    /**
     * Gets the bounds of the shape.
     * @return the bounds of the shape.
     */
    public abstract bound function getBounds():Rectangle2D;

    /**
     * Returns whether this shape contains a given point.
     * @param point a point
     * @return true if the given point lies within the shape.
     */
    public abstract bound function contains(point:Point2D):Boolean;

    public override function toString():String {
        "Stroke: {strokePaint}, fill: {fillPaint}"
    }
}
