package javafxdata;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.StringBuffer;
import javafx.io.http.HttpRequest;
import javafx.scene.Scene;
import javafx.stage.Stage;
import javafx.io.http.HttpHeader;
import javafx.geometry.Insets;
import javafx.scene.control.Button;
import javafx.scene.control.ScrollView;
import javafx.scene.control.TextBox;
import javafx.scene.layout.HBox;
import javafx.scene.layout.LayoutInfo;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;

var inputField:TextBox;
var textDisplay:TextBox;
var r:HttpRequest;
var data = "The data to be written\n";
Stage {
    var scene:Scene;
    title : "HttpRequest Example #5"
    scene: scene = Scene {
        width: 400
        height: 500
        content: [
            VBox {
                width: bind scene.width
                height: bind scene.height
                padding: Insets {
                    top: 3 left:3 bottom: 3 right: 3
                }
                spacing: 3
                content: [
                    ScrollView {
                        fitToWidth: true
                        fitToHeight: true
                        node: textDisplay = TextBox {
                            editable: false
                            multiline: true
                        }
                        layoutInfo: LayoutInfo {
                            vfill: true
                            vgrow: Priority.ALWAYS
                            vshrink: Priority.ALWAYS
                        }
                    }
                    HBox {
                        spacing: 3
                        content: [
                            inputField = TextBox {
                                columns: 20
                            }
                            Button {
                                text: "PUT"
                                disable: bind inputField.rawText.trim() == ""
                                               or (r != null and not r.done)
                                action: function() {
                                    doRequest(inputField.text.trim());
                                }
                            }
                        ]
                    }
                ]
            }
        ]
    }
}
inputField.requestFocus();

function addText(s:String) {
    textDisplay.text = textDisplay.text.concat(s);
}

function doRequest(url:String) {
    r = HttpRequest {
        location: url
        method: HttpRequest.PUT
        headers: [
            HttpHeader {
                name: HttpHeader.CONTENT_TYPE
                value: "text/plain"
            },
            HttpHeader {
                name: HttpHeader.CONTENT_LENGTH
                value: "{data.length()}"
            },
            HttpHeader {
                name: HttpHeader.CONTENT_ENCODING
                value: "UTF-8"
            },
        ]
        onConnecting: function() { addText("onConnecting\n"); }
        onDone: function() { addText("onDone\n") }
        onDoneConnect: function() { addText("onDoneConnect\n"); }
        onDoneHeaders: function() { addText("onDoneHeaders\n"); }
        onDoneRead: function() { addText("onDoneRead\n"); }
        onDoneWrite: function() { addText("onDoneWrite\n"); }
        onError: function(is) { addText("onError: "); displayStream(is)}
        onException: function(ex) { addText("onException: {ex}\n"); }
        onInput: function(is) { addText("onInput: "); countStream(is)}
        onOutput: function(os) { 
            addText("onOutput called\n");
            var bytes = data.getBytes("UTF-8");
            os.write(bytes);
            os.close();
        }
        onRead: function(count) { addText("onRead: {count} bytes\n"); }
        onReading: function() { addText("onReading\n"); }
        onReadingHeaders: function() { addText("onReadingHeaders\n"); }
        onResponseCode: function(code) { addText("onResponseCode: {code}\n"); }
        onResponseHeaders: function(headers) { addText("onResponseHeaders");
                                               showHeaders(headers, r)}
        onResponseMessage: function(msg) { addText("onResponseMessage: {msg}\n"); }
        onStarted: function() { addText("onStarted\n"); }
        onToRead: function(length) { addText("onToRead, length = {length}\n"); }
        onToWrite: function(length) { addText("onToWrite, length = {length}\n"); }
        onWriting: function() { addText("onWriting\n"); }
        onWritten: function(length) { addText("onWritten, length = {length}\n"); }

        var connecting = bind r.connecting on replace { addText("connecting -> {connecting}\n") }
        var done = bind r.done on replace { addText("done -> {done}\n") };
        var doneConnect = bind r.doneConnect on replace { addText("doneConnect -> {doneConnect}\n") };
        var doneHeaders = bind r.doneHeaders on replace { addText("doneHeaders -> {doneHeaders}\n") };
        var doneRead = bind r.doneRead on replace { addText("doneRead -> {doneRead}\n") };
        var doneWrite = bind r.doneWrite on replace { addText("doneWrite -> {doneWrite}\n") };
        var error = bind r.error on replace { addText("error -> {error}\n") };
        var exception = bind r.exception on replace { addText("exception -> {exception}\n") };
        var input = bind r.input on replace { addText("input -> {input}\n") };
        var output = bind r.output on replace { addText("output -> {output}\n") };
        var read = bind r.read on replace { addText("read -> {read}\n") };
        var reading = bind r.reading on replace { addText("reading -> {reading}\n") };
        var readingHeaders = bind r.readingHeaders on replace { addText("readingHeaders -> {readingHeaders}\n") };
        var responseCode = bind r.responseCode on replace { addText("responseCode -> {responseCode}\n") };
        var responseMessage = bind r.responseMessage on replace { addText("responseMessage -> {responseMessage}\n") };
        var started = bind r.started on replace { addText("started -> {started}\n") };
        var toread = bind r.toread on replace { addText("toread -> {toread}\n") };
        var towrite = bind r.towrite on replace { addText("towrite -> {towrite}\n") };
        var writing = bind r.writing on replace { addText("writing -> {writing}\n") };
        var written = bind r.written on replace { addText("written -> {written}\n") };
    }
    r.start();
}

function showHeaders(headers:String[], r:HttpRequest) {
    for (header in headers) {
        addText("    {header}: ");
        addText("{r.getResponseHeaderValue(header)}\n");
    }
}

function countStream(is:InputStream) {
    var count = 0;
    try {
        while (is.read() != -1) {
            count++
        }
    } finally {
        is.close();
    }
    addText("{count} bytes\n");
}

function displayStream(is:InputStream) {
    var r:BufferedReader = new BufferedReader(new InputStreamReader(is));
    var sb:StringBuffer = new StringBuffer();
    var c:Integer;
    while (true) {
        c = r.read();
        if (c == -1) {
            break;
        }
        sb.append(c as Character);
    }
    addText("Error:\n");
    addText(sb.toString());
    is.close();
}




