package javafxdata;

import javafx.scene.CustomNode;
import javafx.scene.image.Image;
import javafx.scene.text.Font;
import javafx.scene.image.ImageView;
import javafx.scene.text.Text;
import javafx.scene.shape.Rectangle;
import javafx.scene.paint.Color;
import javafx.scene.text.TextOrigin;
import javafx.scene.text.TextAlignment;
import javafx.scene.Group;
import javafx.scene.layout.Container;
import javafx.geometry.HPos;
import javafx.geometry.VPos;
import javafx.util.Math;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;
import javafx.scene.effect.DropShadow;

public class InformationNode extends CustomNode {
    public var name:String;
    public var imageURL:String;
    public var message:String;
    public var nameFont = Font { name: "Arial Bold" size: 14 };
    public var messageFont = Font { size: 12 };

    protected override var children = {
        def imageSize = 48;
        def messageWrap = 256;
        def pad = 4;
        def overallWidth = imageSize + messageWrap + 3 * pad;
        var nameNode:Text;
        var messageNode:Text;
        var imageNode:ImageView;

        Container {
            content: [
                Rectangle {
                    width: overallWidth
                    height: bind nameNode.layoutBounds.height + 3 * pad
                            + Math.max(imageSize,
                                       messageNode.layoutBounds.height);
                    arcWidth: 10
                    arcHeight: 10
                    strokeWidth: 2
                    stroke: Color.BLACK
                    fill: LinearGradient {
                        startX: 0.0 endX: 0.0
                        startY: 0.0 endY: 1.0
                        stops: [
                            Stop { offset: 0.0 color: Color.rgb(4, 166, 213) }
                            Stop { offset: 0.3 color: Color.rgb(138, 227, 253) }
                        ]
                    }
                    effect: DropShadow { offsetX: 5 offsetY: 5 }
                }
                nameNode = Text {
                    content: bind name
                    font: bind nameFont
                    fill: Color.WHITE
                    textOrigin: TextOrigin.TOP
                    textAlignment: TextAlignment.CENTER
                }
                messageNode = Text {
                    content: bind message
                    font: bind messageFont
                    wrappingWidth: messageWrap
                    textOrigin: TextOrigin.TOP
                }
                imageNode = ImageView {
                    image: bind Image {
                        url : imageURL
                        backgroundLoading: true
                        width: imageSize
                        height: imageSize
                    }
                }
            ]
            protected override function doLayout() {
                Container.positionNode(nameNode, pad, pad, overallWidth,
                                       nameNode.layoutBounds.height,
                                       HPos.CENTER, VPos.CENTER);
                Container.positionNode(imageNode, pad,
                                       2 * pad + nameNode.layoutBounds.height,
                                       2 * pad + imageSize, imageSize,
                                       HPos.CENTER, VPos.TOP);
                Container.positionNode(messageNode, 2 * pad + imageSize,
                                       2 * pad + nameNode.layoutBounds.height,
                                       messageWrap,
                                       messageNode.layoutBounds.height,
                                       HPos.LEFT, VPos.TOP);
            }
        }
    }
}
