package javafxdata;

import javafx.data.pull.PullParser;
import javafx.io.http.HttpRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

public function getWeather(place:String, onDone:function(String)) {
    var os = new ByteArrayOutputStream();
    var req = HttpRequest {
        location: "http://ws.geonames.org/weatherIcaoJSON?ICAO={place}"
        sink: os
        onDone: function() {
            onDone(processJSONResponse(place, os));
        }
    }
    req.start();
}

function processJSONResponse(place:String,
                             os:ByteArrayOutputStream):String {
    var is = new ByteArrayInputStream(os.toByteArray());
    var country = "(Unreported)";
    var clouds = "(Unreported)";
    var windDir = "(Unreported)";
    var windSpeed = "(Unreported)";
    var temp = "(Unreported)";
    var stationName = "(Unreported)";
    var time = "(Unreported)";
    var parser = PullParser {
        input: is
        documentType: PullParser.JSON
        onEvent: function(evt) {
            var type = evt.type;
            var name = evt.name;
            if (type == PullParser.TEXT) {
                if (name == "countryCode") {
                    country = evt.text;
                } else if (name == "clouds") {
                    clouds = evt.text;
                } else if (name == "windSpeed") {
                    windSpeed = evt.text;
                } else if (name == "temperature") {
                    temp = evt.text;
                } else if (name == "stationName") {
                    stationName = evt.text;
                } else if (name == "datetime") {
                    time = evt.text;
                }
            } else if (type == PullParser.INTEGER) {
                if (name == "windDirection") {
                    windDir = "{evt.integerValue}";
                }
            }
        }
    }
    parser.parse();
    is.close();

    "Weather for airport {place} ({country})\n"
    "Time: {time}\n"
    "Station name is {stationName}\n"
    "Wind {windSpeed} knots from {windDir}\n"
    "Temperature {temp}C\n"
    "Cloud condition: {clouds}\n"
}
