package javafxdata;

import com.sun.javafx.functions.Function0;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import javafx.async.RunnableFuture;
import javafx.lang.FX;

class JavaCustomerFetcher implements RunnableFuture {
    private final String dbURL;
    private final Properties params;
    private final TaskUpdater task;

    JavaCustomerFetcher(String dbURL, Properties params,
                        TaskUpdater task) {
        this.dbURL = dbURL;
        this.params = params;
        this.task = task;
    }
    
    @Override public void run() throws Exception {
        updateProgress(0, 100);

        Class.forName("org.apache.derby.jdbc.ClientDriver");
        Connection conn = null;
        Statement stmt = null;
        List customers = new ArrayList();
        try {
            conn = DriverManager.getConnection(dbURL, params);
            stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery(
                    "SELECT CUSTOMER_ID, NAME from CUSTOMER ORDER BY NAME");
            while (rs.next()) {
                int id = rs.getInt("CUSTOMER_ID");
                String name = rs.getString("NAME");
                customers.add(new JavaCustomer(id, name));
            }
            JavaCustomer[] results = new JavaCustomer[customers.size()];
            setCustomers((JavaCustomer[])customers.toArray(results));
            rs.close();
        } finally {
            if (stmt != null) {
                stmt.close();
            }
            if (conn != null) {
                conn.close();
            }
        }

        updateProgress(100, 100);
    }

    private void updateProgress(final long progress,
                                final long progressMax) {
        FX.deferAction(new Function0() {
            @Override public Void invoke() {
                task.setProgress(progress, progressMax);
                return null;
            }
        });
    }

    private void setCustomers(final JavaCustomer[] results) {
        FX.deferAction(new Function0() {
            @Override public Void invoke() {
                task.setReturnValue(results);
                return null;
            }
        });
    }
}
