package javafxdata;

import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.geometry.HPos;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;
import javafx.scene.paint.Color;
import javafx.scene.layout.Container;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafxdata.TwitterEntry;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.geometry.Insets;
import javafx.scene.control.TextBox;
import javafx.scene.layout.LayoutInfo;
import javafx.scene.layout.Priority;
import javafx.scene.control.ScrollBar;
import javafx.scene.layout.ClipView;
import javafx.scene.Cursor;

var fetchingEntries:Boolean;
var targetContainer:Container;
Stage {
    title: "Twitter Atom Search"
    width: 380
    height: 450
    var scene:Scene;
    scene: scene = Scene {
        cursor: bind if (fetchingEntries) Cursor.WAIT
                     else Cursor.DEFAULT
        fill: LinearGradient {
            startX: 0.0 endX: 0.0
            startY: 0.0 endY: 1.0
            stops: [
                Stop { offset: 0.0 color: Color.DARKBLUE }
                Stop { offset: 1.0 color: Color.LIGHTBLUE }
            ]
        }
        content: [
            VBox {
                width: bind scene.width
                height: bind scene.height
                var textBox:TextBox;
                padding: Insets {
                    top: 3 left: 3 bottom: 3 right: 3
                }
                spacing: 3
                nodeHPos: HPos.CENTER
                content: [
                    Text {
                        content: "Twitter Atom Search"
                        fill: Color.WHITE
                        font: Font { name: "Arial Bold" size: 14 }
                    }
                    HBox {
                        spacing: 4
                        content: [
                            Text {
                                content: "Search Term: "
                                fill: Color.rgb(188, 194, 252)
                                font: Font { name: "Arial Bold" size: 16 }
                            }
                            textBox = TextBox {
                                columns: 16
                                disable: bind fetchingEntries
                                action: function() {
                                    doSearch(textBox.text.trim());
                                }
                            }
                        ]
                    }
                    HBox {
                        var clipView:ClipView;
                        layoutInfo: LayoutInfo {
                            vfill: true
                            vgrow: Priority.ALWAYS
                        }
                        content: [
                            clipView = ClipView {
                                node: targetContainer = VBox {
                                    nodeHPos: HPos.CENTER
                                }
                                pannable: false
                                layoutInfo: LayoutInfo {
                                    vfill: true
                                    hfill: true
                                    hshrink: Priority.ALWAYS
                                    hgrow: Priority.ALWAYS
                                    vgrow: Priority.ALWAYS
                                }
                            }
                            ScrollBar {
                                vertical: true
                                max: bind clipView.maxClipY
                                blockIncrement: 64
                                override var value on replace {
                                    clipView.clipY = value
                                }
                                layoutInfo: LayoutInfo {
                                    vfill: true
                                    hgrow: Priority.NEVER
                                    vshrink: Priority.ALWAYS
                                    vgrow: Priority.ALWAYS
                                }
                            }
                        ]
                    }
                ]
            }
        ]
    }
}


function doSearch(query:String) {
    fetchingEntries = true;
    delete targetContainer.content;
    TwitterAtomSearchClient.doTwitterAtomSearch(query, 50,
         function(results:TwitterEntry[]):Void {
             delete targetContainer.content;
             insert for (entry in results) {
                InformationNode {
                    name: entry.user
                    imageURL: entry.imageURL
                    message: "{entry.time}\n{entry.content}"
                }
             } into targetContainer.content;
             fetchingEntries = false;
         }
    );
}
