package javafxdata;

import javafx.data.pull.PullParser;
import javafx.io.http.HttpRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import javafx.io.http.HttpStatus;

public function getInformITTimeline(onDone:function(TwitterEntry[])) {
    var entries:TwitterEntry[] = null;
    var os = new ByteArrayOutputStream();
    var req:HttpRequest = HttpRequest {
        location: "http://api.twitter.com/1/statuses/user_timeline/informit.xml"
        sink: os
        onDone: function() {
            if (req.responseCode == HttpStatus.OK) {
                insert timelineResponse(os) into entries;
            } else {
                insert TwitterEntry {
                    content: "Failed to get content\n"
                             "{req.responseMessage}"
                } into entries;
            }
            onDone(entries);
        }
    };
    req.start();
}

function timelineResponse(os:ByteArrayOutputStream):TwitterEntry[] {
    var entries:TwitterEntry[];
    var is = new ByteArrayInputStream(os.toByteArray());
    var parser = PullParser {
        input: is
        documentType: PullParser.XML
        onEvent: function(evt) {
            var name = evt.qname.name;
            if (evt.type == PullParser.START_ELEMENT
                   and name == "status") {
                user = imageURL = time = content = "";
            } else if (evt.type == PullParser.TEXT) {
                if (name == "screen_name") {
                    user = evt.text;
                } else if (name == "profile_image_url") {
                    imageURL = evt.text;
                } else if (name == "created_at"
                            and evt.level == 2) {
                    time = evt.text;
                } else if (name == "text") {
                    content = evt.text;
                }
            } else if (evt.type == PullParser.END_ELEMENT) {
                if (name == "status") {
                    insert TwitterEntry {
                        user: user
                        imageURL: imageURL
                        content: content
                        time: time
                    } into entries;
                }
            }
        }
        var user;
        var imageURL;
        var content;
        var time;
    }
    parser.parse();
    is.close();

    entries
}
