package javafxexprfuncliterals;

import java.lang.System;
import java.util.Random;
import javafx.scene.Scene;
import javafx.scene.shape.Rectangle;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import javafx.util.Math;

// A function variable
var fn:function(arg:Number):Number;

// A function variable, unnamed argument
var fnNoArgName:function(:Number):Number;

// One way to generate a random number.
function random1(max:Number):Number {
    print("random1: ");
    return Math.random() * max;
}

// Another way to generate a random number.
var seededRandom = new Random();
function random2(max:Number):Number {
    print("random2: ");
    return seededRandom.nextDouble() * max;
}


// Variable definition
var getRandom:function(:Number):Number;

// Assign the first function to the variable and invoke it
getRandom = random1;
println("{getRandom(10)}");

// Assign the second function to the variable and invoke it
getRandom = random2;
println("{getRandom(10)}");

    
function timing(f:function(:Number):Number, arg:Number, name:String):Number {
    print("Calling {name}...");
    var start:Number = System.nanoTime();
    var result = f(arg);
    var time:Number = System.nanoTime() - start;
    println("Result is {result}, time {time} nanoseconds");
    result
}

timing(random1, 4, "random1");
timing(random2, 4, "random2");

function choose():function(:Number):Number {
    if (Math.random() >= 0.5) random1 else random2
}

for (i in [1..10]) {
    def f:function(:Number):Number = choose();
    println(f(4));
}

for (i in [1..10]) {
    println(choose()(4));
}

// Assigning a function to a function attribute.
function stageClose() {
    println("'{stage1.title}' closing");
}

def stage1:Stage = Stage {
    title: "Close Action #1"
    width: 400 height: 400 x: 100 y: 10 visible: true
    onClose: stageClose
}

// Anonymous function
def stage2:Stage = Stage {
    title: "Close Action #2"
    width: 400 height: 400 x: 500 y: 10 visible:true
    onClose: function() {
                println("'{stage2.title}' closing");
             }
}
    
// Anonymous function assigned to a variable    
var fn1 = function():Number {
                  return Math.random();
           };
println("A random number: {fn1()}");

// Returning an anonymous function
function getRandomSource():function(:Number):Number {
    return function (max:Number):Number {
        return Math.random() * max;
    }
}
println("Random from anonymous function: {getRandomSource()(4)}");

var stage:Stage = Stage {
    title: "Mouse events"
    visible: true
    width: 600
    height: 400
    scene: Scene {
        content: [
            Rectangle {
                fill: Color.YELLOW
                x: 0
                y: 0
                width: bind stage.width / 2
                height: bind stage.height
                onMouseMoved: function(evt:MouseEvent) {
                                 println("Mouse at {evt.x}, {evt.y}");
                              }
            },
            Rectangle {
                fill: Color.GREEN
                x: bind stage.width / 2
                y: 0
                width: bind stage.width / 2
                height: bind stage.height
                onMouseMoved: function(evt) {
                                 println("Mouse at {evt.x}, {evt.y}");
                              }
            }
        ]
    }
}
