package javafxreflection;

import java.lang.Object;
import javafx.reflect.FXContext;
import javafx.reflect.FXLocal;
import javafx.reflect.FXSequenceType;
import javafx.reflect.FXSequenceValue;
import javafx.reflect.FXValue;

// Declare a sequence
var sequence = ["A", "B", "C", "D"];

// Get the value of the sequence.
var ctx = FXContext.getInstance();
var classType = ctx.findClass("javafxreflection.SequenceReflection");
var sequenceMember = classType.getVariable("sequence");
var value = sequenceMember.getValue(null);
var seqValue = value as FXSequenceValue;

// Get the sequence type
var seqType = seqValue.getType() as FXSequenceType;
var elementType = seqType.getComponentType();
println("Sequence type is {seqType}");
println("Element type is {elementType}");

// Get and print the elements
println("Number of elements: {seqValue.getItemCount()}");
for (i in [0..<seqValue.getItemCount()]) {
    var elementValue = seqValue.getItem(i);
    var actualValue = getActualValue(elementValue);
    println("  Element {i}: {actualValue}");
}

// Sequence creation.
var intType = ctx.getIntegerType();
var values:FXValue[] = [ctx.mirrorOf(1), ctx.mirrorOf(2), ctx.mirrorOf(3)];
value = ctx.makeSequence(intType, values);
printSequenceValue(value as FXSequenceValue);

var builder = ctx.makeSequenceBuilder(intType);
builder.append(ctx.mirrorOf(1));
builder.append(ctx.mirrorOf(2));
builder.append(ctx.mirrorOf(3));
value = builder.getSequence();
printSequenceValue(value as FXSequenceValue);

function printSequenceValue(seqValue:FXSequenceValue) {
    println("Number of elements: {seqValue.getItemCount()}");
    for (i in [0..<seqValue.getItemCount()]) {
        var elementValue = seqValue.getItem(i);
        var actualValue = getActualValue(elementValue);
        println("  Element {i}: {actualValue}");
    }
}
function getActualValue(value:FXValue):Object {
    var actualValue = null;
    if (value instanceof FXLocal.Value) {
        actualValue = (value as FXLocal.Value).asObject();
    }
    return actualValue;
}
