package javafxreflection;

import java.lang.Object;
import javafx.reflect.FXContext;
import javafx.reflect.FXFunctionValue;
import javafx.reflect.FXLocal;
import javafx.reflect.FXValue;
import javafxreflection.FunctionsAndVariables;

var ctx = FXContext.getInstance();

// Create an object instance.
var controlClassType = ctx.findClass("javafx.scene.control.TextBox");
var objectValue = controlClassType.newInstance();
var localObjectValue = objectValue as FXLocal.ObjectValue;
var textBox = localObjectValue.asObject();
println("Control is {textBox}");

function getActualValue(value:FXValue):Object {
    var actualValue = null;
    if (value instanceof FXLocal.Value) {
        actualValue = (value as FXLocal.Value).asObject();
    }
    return actualValue;
}

// Get the value of a script variable.
var classType = ctx.findClass("javafxreflection.FunctionsAndVariables");
var scriptVarMember = classType.getVariable("scriptVar");
var value = scriptVarMember.getValue(null);
var actualValue = getActualValue(value);
println("Value of scriptVar is '{actualValue}'");

// Change the value of a script variable.
var newValue = "New Value";
var newObjectValue = (ctx as FXLocal.Context).mirrorOf(newValue);
scriptVarMember.setValue(null, newObjectValue);

// Read back the changed value of a script variable.
value = scriptVarMember.getValue(null);
actualValue = getActualValue(value);
println("New value of scriptVar is '{actualValue}'");

// Get the value of an instance variable
var instance = FunctionsAndVariables {};
var instanceObjectValue = (ctx as FXLocal.Context).mirrorOf(instance);
var instanceVarMember = classType.getVariable("instanceVar");
value = instanceVarMember.getValue(instanceObjectValue);
actualValue = getActualValue(value);
println("Value of instanceVar is '{actualValue}'");

// Change the value of an instance variable.
var newIntValue = ctx.mirrorOf(32);
instanceVarMember.setValue(instanceObjectValue, newIntValue);

// Read back the changed value of an instance variable
value = instanceVarMember.getValue(instanceObjectValue);
actualValue = getActualValue(value);
println("New value of instanceVar is '{actualValue}'");

// Invoking a static function from an FXFunctionValue
var aFunctionMember = classType.getVariable("aFunction");
value = aFunctionMember.getValue(null);
println(value);

var functionValue = value as FXFunctionValue;
var arg1 = ctx.mirrorOf(1);
var arg2 = ctx.mirrorOf(2);
value = functionValue.apply([arg1, arg2]);
actualValue = getActualValue(value);
println("Result of invoking add() via FXFunctionValue is {actualValue}");

// Invoke the instance function multiply(Integer, Integer)
var integerType = ctx.getIntegerType();
var intMultMember = classType.getFunction("multiply", [integerType, integerType]);
objectValue = (ctx as FXLocal.Context).mirrorOf(instance);
functionValue = intMultMember.asFunction(objectValue);

arg1 = ctx.mirrorOf(2);
arg2 = ctx.mirrorOf(3);
value = functionValue.apply([arg1, arg2]);
actualValue = getActualValue(value);
println("Result of multiply() via FXFunctionMember is {actualValue}");

// Invoke a named function of an FXObjectValue with given arguments
value = objectValue.invoke(intMultMember,
                           [ctx.mirrorOf(4), ctx.mirrorOf(5)]);
actualValue = getActualValue(value);
println("Result of multiply() via FXObjectValue #1: {actualValue}");

value = objectValue.invoke("multiply",
                           [ctx.mirrorOf(4), ctx.mirrorOf(5)]);
actualValue = getActualValue(value);
println("Result of multiply() via FXObjectValue #2: {actualValue}");



