package javafxsequences;

import java.util.Comparator;
import java.util.GregorianCalendar;
import javafx.util.Sequences;

// Compare by identity
var date1A:GregorianCalendar = new GregorianCalendar(2008, 10, 20);
var date1B:GregorianCalendar = new GregorianCalendar(2008, 10, 20);
var date2A:GregorianCalendar = new GregorianCalendar(2008, 10, 21);
var date2B:GregorianCalendar = new GregorianCalendar(2008, 10, 21);

var datesA = [date1A, date2A];
var datesB = [date1B, date2B];
println("datesA == datesB? {datesA == datesB}");
println("datesA/datesB same? {Sequences.isEqualByContentIdentity(datesA, datesB)}");

var datesC = [date1A, date2A];
println("datesA/datesC same? {Sequences.isEqualByContentIdentity(datesA, datesC)}");
println("datesA/datesA same? {Sequences.isEqualByContentIdentity(datesA, datesA)}");

// Search for elements
var strings = ["A", "B", "C", "A", "B"];

var index = Sequences.indexOf(strings, "A");
println('Index of "A": {index}');

index = Sequences.nextIndexOf(strings, "A", index + 1);
println('Second index of "A": {index}');

index = Sequences.nextIndexOf(strings, "A", index + 1);
println('Third index of "A": {index}');

// Maximum and minumum of a set of numbers and strings.
var ints = [99, 88, 1, 23, 44];
var maxInt:Integer = Sequences.max(ints) as Integer;
var minInt:Integer = Sequences.min(ints) as Integer;
println("ints: max is {maxInt}, min is {minInt}");

var strs = ["Abcd", "A", "AA", "ZZ", "Z"];
var maxStr:String = Sequences.max(strs) as String;
var minStr:String = Sequences.min(strs) as String;
println("strings: max is {maxStr}, min is {minStr}");

var comp:Comparator = Comparator {
    override function compare(o1: Object, o2:Object): Integer {
        var s1 = o1 as String;
        var s2 = o2 as String;
        var l1 = s1.length();
        var l2 = s2.length();
        if (l1 == l2) {
            return s1.compareTo(s2);
        } else if (l1 < l2) {
            return -1;
        } else {
            return 1;
        }
    }
    
    override function equals(o:Object): Boolean {
        return o == comp;
    }
};

println("strings with comparator: max is {Sequences.max(strs, comp)}");
println("strings with comparator: min is {Sequences.min(strs, comp)}");

// Sorting of Comparables.
var sortedInts:Integer[] = Sequences.sort(ints) as Integer[];
println("Sorted ints: {sortedInts.toString()}");
println("ints after sort: {ints.toString()}");

var sortedStrs:String[] = Sequences.sort(strs) as String[];
println("Sorted strings: {sortedStrs.toString()}");
println("strings after sort: {strs.toString()}");

// Sorting using a Comparator.
sortedStrs = Sequences.sort(strs, comp) as String[];
println("Sorted strings: {sortedStrs.toString()}");

// Binary search for an element that is present.
var intIndex = Sequences.binarySearch(sortedInts, 44);
println("Sorted ints: {sortedInts.toString()}, 44 is at index {intIndex}");

// Binary search for an element that is not present.
intIndex = Sequences.binarySearch(sortedInts, 55);
println("Sorted ints: {sortedInts.toString()}, 55 is at index {intIndex}");
var insertIndex = -intIndex - 1;
insert 55 before sortedInts[insertIndex];
println("Sorted ints after insert: {sortedInts.toString()}");

// Shuffling a sequence.
var shuffledInts:Integer[] = Sequences.shuffle(ints) as Integer[];
println("First shuffle: {shuffledInts.toString()}");

shuffledInts = Sequences.shuffle(ints) as Integer[];
println("Second shuffle: {shuffledInts.toString()}");

shuffledInts = Sequences.shuffle(ints) as Integer[];
println("Third shuffle: {shuffledInts.toString()}");
