package javafxsequences;

import java.lang.Comparable;

import java.lang.Object;
import java.util.Date;
import javafx.scene.Node;
import javafx.scene.shape.Rectangle;
import javafx.util.Math;

// Creating sequences.
var integers: Integer[] = [1, 2, 3, 4, 5];
var numbers: Number[] = [1.1, 2.2, 3.3, 4.4, 5.5];
var strings: String[] = ["The", "cat", "sat", "on", "the", "mat"];
var objects: Object[] = ["String", 1, 2.4, new Date()];
var emptySequence1: Integer[] = [];
var emptySequence2: Integer[] = null;

// Inferred types
var integers2 = [1, 2, 3, 4, 5];
var numbers2 = [1.1, 2.2, 3.3, 4.4, 5.5];
var strings2 = ["The", "cat", "sat", "on", "the", "mat"];
var objects2 = ["String", 1, 2.4, new Date()];

// Sequence printing
println("Integers: {integers}");
println("Numbers: {numbers}");
println("Strings: {strings}");
println("Objects: {objects}");
println("Empty 1: {emptySequence1}");
println("Empty 2: {emptySequence2}");

// Printing sequences for debugging
println("toString() integers: {integers.toString()}");
println("toString() numbers: {numbers.toString()}");
println("toString() strings: {strings.toString()}");
println("toString() objects: {objects.toString()}");
println("toString() Empty 1: {emptySequence1.toString()}");
println("toString() Empty 2: {emptySequence2.toString()}");

// Singleton sequence.
var singleton: Integer[] = 1;
println("Singleton sequence: {singleton.toString()}");

// Flattening of nested sequences
var flattened: Integer[] = [1, [2, 3], [4, 5, 6]];
println("Flattened sequence: {flattened.toString()}");

// Out-of-range indices
println("integers[-1] = {integers[-1]}, integers[10] = {integers[10]}");
println("numbers[-1] = {numbers[-1]}, numbers[10] = {numbers[10]}");
println("strings[-1] = {strings[-1]}, strings[10] = {strings[10]}");
println("objects[-1] = {objects[-1]}, objects[10] = {objects[10]}");

// Ranges
var range1: Integer[] = [1..5];
println("range1: {range1.toString()}");

var range2: Integer[] = [1..5 step 2];
println("range2: {range2.toString()}");

var range3: Integer[] = [5..1 step -1];
println("range3: {range3.toString()}");

var range4: Number[] = [1..5 step 0.5];
println("range4: {range4.toString()}");

var range5: Integer[] = [1..<5];
println("range5: {range5.toString()}");

var range6: Integer[] = [5..<1 step -1];
println("range6: {range6.toString()}");

var bound1: Integer = (Math.random() * 10) as Integer;
var bound2: Integer = (Math.random() * 10) as Integer;
var range7: Integer[] = [Math.min(bound1, bound2)..
                         Math.max(bound1, bound2)];
println("range7: {range7.toString()}");

// Reversing sequences
var range: Integer[] = [1..10];
var reverseRange: Integer[] = reverse range;
println("Range: {range.toString()}");
println("Reverse of range: {reverseRange.toString()}");

// Equality and copying of sequences
var seq1: Integer[] = [1..5];
var seq2: Integer[] = [1..5];
println("seq1 == seq2? {seq1 == seq2}");

var seq3: Integer[] = [1..6];
println("seq1 == seq3? {seq1 == seq3}");

var seq1Copy: Integer[] = seq1;
println("seq1 == seq1Copy? {seq1 == seq1Copy}");

insert 6 into seq1;
println("After insert, seq1: { seq1.toString()}, "
         "seq1Copy: {seq1Copy.toString()}");

// Assignment of sequences
var rectangles: Rectangle[] = [
    Rectangle{ },
    Rectangle{ }];
var nodes: Node[] = rectangles;
println("{nodes.toString()}");
var comparables: Comparable[] = ["A", "B"];

// Size of a sequence
var tenInts: Integer[] = [1..10];
var noInts: Integer[] = [];
println("Size of tenInts: {sizeof tenInts}, size of noInts: {sizeof noInts}");

// Sequence indexing
var int0: Integer = tenInts[0];
var int9: Integer = tenInts[9];
println("into: {int0}, int9: {int9}");

// Sequence of one element
var seqOfOne: Integer[] = tenInts[4];
println("seqOfOne: {seqOfOne.toString()}");

// Range expression
var subseq1: Integer[] = tenInts[0..2];
var subseq2: Integer[] = tenInts[sizeof tenInts - 3..
                                 sizeof tenInts - 1];
println("subseq1: {subseq1.toString()}, "
        "subseq2: {subseq2.toString()}");

var subseq3: Integer[] = tenInts[7..<sizeof tenInts - 1];
println("subseq3: {subseq3.toString()}");

var subseq4: Integer[] = tenInts[7..120];
var subseq5: Integer[] = tenInts[-7..1];
println("subseq4: {subseq4.toString()}, subseq5: {subseq5.toString()}");

var subseq6: Integer[] = tenInts[7..7];
println("subseq6: {subseq6.toString()}");
var subseq7: Integer[] = tenInts[7..6];
println("subseq7: {subseq7.toString()}");
var subseq8: Integer[] = tenInts[0..<];
println("subseq8: {subseq8.toString()}");
var subseq9: Integer[] = tenInts[0..];
println("subseq9: {subseq9.toString()}");

// Queries
var greater5: Integer[] = tenInts[n | n > 5];
println("greater5: {greater5.toString()}");
var greater5less9: Integer[] = tenInts[n | n > 5 and n < 9];
println("greater5less9: {greater5less9.toString()}");

var tenStrings: String[] = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"];
println("tenStrings[n | indexof n > 1]: {tenStrings[n | indexof n > 1].toString()}");

// Replacement by index.
tenInts[4] = 24;
println("tenInts after replacing index 4: {tenInts.toString()}");

// Insert at end.
tenInts = [1..10];
insert 32 into tenInts;
println("tenInts after insert 32: {tenInts.toString()}");

tenInts = [1..10];
insert [21, 22] into tenInts;
println("tenInts after insert [21, 22]: {tenInts.toString()}");

tenInts = [1..10];
insert null into tenInts;
println("tenInts after insert null: {tenInts.toString()}");

tenInts = [1..10];
insert tenInts[
    n | n > 7] into tenInts;
println("tenInts after query insert: {tenInts.toString()}");

// Insert before and after
tenInts = [1..10];
insert 6 before tenInts[1];
println("tenInts after insert before second: {tenInts.toString()}");

tenInts = [1..10];
insert [100, 101] after tenInts[1];
println("tenInts after insert [100, 101] after second: {tenInts.toString()}");

// Removal of elements
tenInts = [1..10];
delete tenInts[2];
println("tenInts after delete of index 2: {tenInts.toString()}");

tenInts = [1..10];
delete tenInts[5..<];
println("tenInts after delete of [5..<]: {tenInts.toString()}");

var moreInts: Integer[] = [[1..5], [1..5]];
println("moreInts: {moreInts.toString()}");
delete 3 from moreInts;
println("moreInts after delete 3: {moreInts.toString()}");

// Range replacement
tenInts = [1..10];
tenInts[0..1] = [20, 21];
println("tenInts after replace of [0..1]: {tenInts.toString()}");

tenInts = [1..10];
tenInts[0..1] = [20, 21, 22];
println("tenInts after replace of [0..1] with three values: {tenInts.toString()}");

tenInts = [1..10];
tenInts[0..1] = 20;
println("tenInts after replace of [0..1] with one value: {tenInts.toString()}");

tenInts = [1..10];
tenInts[0..1] = [];
println("tenInts after replace of [0..1]: {tenInts.toString()}");

tenInts = [1..10];
tenInts[5..4] = [21, 22, 23];
println("tenInts after replace of [5..4]: {tenInts.toString()}");

