package javafxshapes;

import javafx.scene.input.MouseEvent;
import javafx.scene.Scene;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.QuadCurve;
import javafx.stage.Stage;

var start:Circle;
var end:Circle;
var control:Circle;

var startX = 20.0 on replace { printValues() };
var startY = 100.0 on replace { printValues() };
var endX = 180.0 on replace { printValues() };
var endY = 100.0 on replace { printValues() };
var controlX = 100.0 on replace { printValues() };
var controlY = 50.0 on replace { printValues() };

function printValues() {
    println("startX: {startX}, startY: {startY}, "
            "endX: {endX}, endY: {endY}, "
            "controlX: {controlX}, controlY: {controlY}");
}

class CircleDragger {
    public-init var circle:Circle;
    var startX:Number;
    var startY:Number;

    function onMousePressed(evt:MouseEvent):Void {
        startX = circle.centerX;
        startY = circle.centerY;
    }

    function onMouseDragged(evt:MouseEvent):Void {
        circle.centerX = startX + evt.dragX;
        circle.centerY = startY + evt.dragY;
    }

    postinit {
        circle.onMousePressed = onMousePressed;
        circle.onMouseDragged = onMouseDragged;
    }
}

Stage {
    title: "Quadratic Curve"
    scene: Scene {
        width: 200
        height: 200
        content: [
            QuadCurve {
                startX: bind startX
                startY: bind startY
                endX: bind endX
                endY: bind endY
                controlX: bind controlX
                controlY: bind controlY
                fill: null
                stroke: Color.RED
            }
            start = Circle {
                centerX: bind startX with inverse
                centerY: bind startY with inverse
                radius: 3
            }
            end = Circle {
                centerX: bind endX with inverse
                centerY: bind endY with inverse
                radius: 3
            }
            control = Circle {
                centerX: bind controlX with inverse
                centerY: bind controlY with inverse
                radius: 3
            }
        ]
    }
}

CircleDragger { circle: start }
CircleDragger { circle: end }
CircleDragger { circle: control }
