package javafxshapes;

import javafx.geometry.Rectangle2D;
import javafx.scene.Cursor;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.Scene;
import javafx.stage.Stage;

var scene:Scene;
var image:Image;
var imageView:ImageView;
var viewportX:Number;
var viewportY:Number;
var viewport:Rectangle2D = bind Rectangle2D {
                                minX: viewportX
                                minY: viewportY
                                width: scene.width
                                height: scene.height
                           };

function updateViewport(deltaX:Number, deltaY:Number):Void {
    var newX = viewportX + deltaX;
    var newY = viewportY + deltaY;
    if (newX + viewport.width > image.width) {
        newX = image.width - viewport.width;
    }
    if (newX < 0) {
        newX = 0;
    }
    if (newY + viewport.height > image.height) {
        newY = image.height - viewport.height;
    }
    if (newY < 0) {
        newY = 0;
    }
    viewportX = newX;
    viewportY = newY;
}

Stage {
    title: "Images 9"
    scene: scene = Scene {
        width: 500
        height: 500
        content: [
            imageView = ImageView {
                image: image = Image {
                    backgroundLoading: true
                    url: "{__DIR__}images/Portsmouth.JPG"
                }
                viewport: bind viewport

                var lastX:Number;
                var lastY:Number;
                onMousePressed: function(evt) {
                                    lastX = evt.x; lastY = evt.y;
                                    imageView.cursor = Cursor.MOVE
                                }
                onMouseDragged: function(evt) {
                                    var deltaX = lastX - evt.x;
                                    var deltaY = lastY - evt.y;
                                    lastX = evt.x; lastY = evt.y;
                                    updateViewport(deltaX, deltaY);
                                }
                onMouseReleased: function(evt) {
                                     imageView.cursor = Cursor.DEFAULT
                                 }
            }
        ]
    }
}
