package javafxexample;

import java.lang.System;
import java.util.Random;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.paint.Color;
import javafx.scene.Scene;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;

// Static user interface setup

// The scene that contains the snow.
var scene:Scene;

Stage {
    title: "JavaFX Snow Storm"
    resizable: false
    scene: scene = Scene {
        content: [
            ImageView {
                image: Image {
                    url: "{__DIR__}SnowScene.png"
                };
            },
            Text {
                x: 25
                y: bind scene.height - 10
                font: Font {
                    size: 20
                }
                content: bind "Snowflakes: {sizeof snowflakes}"
            }
        ]
    }
}

// Animation

// The snowflakes
var snowflakes:Circle[];

// Time interval for new snow.
def newSnowTime = 500ms;

// Number of snowflakes per row
def snowflakesPerLine = 5;

// Random number generator
def rand = new Random(System.currentTimeMillis());

// Periodically add snow
def addSnowTimeline = Timeline {
    repeatCount: Timeline.INDEFINITE
    keyFrames: [
        KeyFrame {
            time: newSnowTime
            action: addNewSnow
        }
    ]
};
addSnowTimeline.play();

// Adds new snow to the scene.
function addNewSnow() {
    var sectionWidth =
           (scene.width + snowflakesPerLine - 1) / snowflakesPerLine;
    for (i in [0..<snowflakesPerLine]) {
        var x = sectionWidth * i + rand.nextInt(sectionWidth);
        var snowflake = Circle {
            centerX: x
            centerY: 0
            radius: 2
            cache: true
            fill: Color.WHITE
        }
        insert snowflake into scene.content;
        insert snowflake into snowflakes;
    }
}
// Animating the snowflakes

// Time interval for moving the snow.
def moveSnowTime = 100ms;

// How far the snow falls in each frame
def snowFallDistance = 4;

// Drift distance
def snowDriftDistance = 4;

// Periodically move snow
def moveSnowTimeline = Timeline {
    repeatCount: Timeline.INDEFINITE
    keyFrames: [
        KeyFrame {
            time: moveSnowTime
            action: moveSnow
        }
    ]
}
moveSnowTimeline.play();

// Animates the snow.
function moveSnow() {
    for (snowflake in snowflakes) {
        // Move the snowflake down the screen
        snowflake.centerY += snowFallDistance;
        var random = rand.nextDouble();

        // Add some random sideways movement
        if (random > 0.7) {
            snowflake.centerX += snowDriftDistance;
        } else if (random < 0.3) {
            snowflake.centerX -= snowDriftDistance;
        }

        // If the snowflake is off the screen, remove it
        if (snowflake.centerY >= scene.height) {
            delete snowflake from scene.content;
            delete snowflake from snowflakes;
        }
    }
}




