package javafxexample2;

import javafx.scene.image.Image;
import javafx.scene.paint.Color;
import javafx.scene.Scene;
import javafx.stage.Stage;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.scene.image.ImageView;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import java.lang.System;
import java.util.Random;
import javafx.geometry.Bounds;
import javafx.geometry.BoundingBox;
import javafx.util.Math;

// Static user interface setup

// The scene that contains the snow.
var scene:Scene;

// The background image
def image:Image = Image {
                    url: "{__DIR__}SnowScene.png"
                  };

// The ImageView that holds the background image
var imageView:ImageView;

Stage {
    title: "JavaFX Snow Storm 2"
    resizable: true
    scene: scene = Scene {
        width: image.width
        height: image.height
        fill: Color.BLACK
        content: [
            imageView = ImageView {
                layoutX: bind (scene.width - image.width) / 2
                layoutY: bind (scene.height - image.height) / 2
                image: image
            },
            Text {
                x: 25
                y: bind scene.height - 10
                font: Font {
                    size: 20
                }
                fill: Color.YELLOW
                content: bind "Snowflakes: {sizeof snowflakes}"
            }
        ]
    }
}

// Animation

// Radius of snowflake
def snowflakeRadius = 2;

// The snowflakes
var snowflakes:Circle[];

// Time interval for new snow.
def newSnowTime = 500ms;

// Number of snowflakes per row
def snowflakesPerLine = 5;

// Random number generator
def rand = new Random(System.currentTimeMillis());

// Gets the bounds of the area occupied by the snow
bound function getSnowBounds():Bounds {
    var imageBounds = imageView.boundsInParent;
    var bounds = BoundingBox {
        width: Math.min(scene.width, imageBounds.width)
        height: Math.min(scene.height, imageBounds.height)
        minX: Math.max(0, imageBounds.minX)
        minY: Math.max(0, imageBounds.minY)
    }
}

// Area occupied by the image
def imageBounds:Bounds = bind getSnowBounds() on replace {
    removeSnow();
}

// Periodically add snow
def addSnowTimeline = Timeline {
    repeatCount: Timeline.INDEFINITE
    keyFrames: [
        KeyFrame {
            time: newSnowTime
            action: addNewSnow
        }
    ]
};
addSnowTimeline.play();

// Adds new snow to the scene.
function addNewSnow() {
    var sectionWidth =
           (imageBounds.width + snowflakesPerLine - 1) / snowflakesPerLine;
    for (i in [0..<snowflakesPerLine]) {
        var x = sectionWidth * i + rand.nextInt(sectionWidth);
        var snowflake = Circle {
            centerX: imageBounds.minX + x
            centerY: imageBounds.minY
            radius: snowflakeRadius
            cache: true
            fill: Color.WHITE
        }
        insert snowflake into scene.content;
        insert snowflake into snowflakes;
    }
}

// Animating the snowflakes

// Time interval for moving the snow.
def moveSnowTime = 100ms;

// How far the snow falls in each frame
def snowFallDistance = 4;

// Drift distance
def snowDriftDistance = 4;

// Periodically move snow
def moveSnowTimeline = Timeline {
    repeatCount: Timeline.INDEFINITE
    keyFrames: [
        KeyFrame {
            time: moveSnowTime
            action: moveSnow
        }
    ]
}
moveSnowTimeline.play();

// Animates the snow.
function moveSnow() {
    for (snowflake in snowflakes) {
        // Move the snowflake down the screen
        snowflake.centerY += snowFallDistance;
        var random = rand.nextDouble();

        // Add some random sideways movement
        var x = snowflake.centerX;
        if (random > 0.7) {
            x += snowDriftDistance;
        } else if (random < 0.3) {
            x -= snowDriftDistance;
        }
        x = Math.max(imageBounds.minX + snowflakeRadius, x);
        snowflake.centerX = Math.min(x, imageBounds.maxX - snowflakeRadius);

        // If the snowflake is off the screen, remove it
        if (snowflake.centerY >= imageBounds.maxY) {
            delete snowflake from scene.content;
            delete snowflake from snowflakes;
        }
    }
}

// Removes all snow.
function removeSnow():Void {
    for (snowflake in snowflakes) {
        delete snowflake from scene.content;
    }
    snowflakes = [];
}






