package javafxbinding;

import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import javafx.util.Math;
import javafx.scene.control.Slider;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;

var radius = 10.0;
var showArea = true;
var font = Font.font("Arial", FontWeight.BOLD, 16);

function computeArea(radius:Integer):Integer {
    Math.round(Math.PI * radius * radius) as Integer
}

function computeCircumference(radius:Integer):Integer {
    Math.round(2 * Math.PI * radius) as Integer;
}

bound function getAreaOrCircumference(radius:Integer) {
    if (showArea) "Area: {computeArea(radius)}"
             else "Circumference: {computeCircumference(radius)}"
}

Stage {
    title: "A Bound Function"
    visible: true
    width: 300
    height: 220
    resizable: false
    scene: Scene {
              fill: Color.WHITE
              content: [
                  Group {
                      var toggleGroup = ToggleGroup {}
                      content: [
                              RadioButton {
                                  text: "Show Area"
                                  selected: bind showArea with inverse
                                  toggleGroup: toggleGroup
                                  translateX: 20                                  
                              },
                              RadioButton {
                                  text: "Show Circumference"
                                  toggleGroup: toggleGroup
                                  translateX: 130
                              }
                          ]
                  }
                  Group {
                    translateY: 20
                    content: [
                        Circle {
                            centerX: 60
                            centerY: 60
                            radius: bind radius
                            fill: Color.GREEN
                        },
                        Text {
                             content: bind "Radius: {radius}"
                             font: font
                             x: 120
                             y: 50
                         },
                         Text {
                             content: bind getAreaOrCircumference(radius)
                             font: font
                             x: 120
                             y: 80
                         },
                    ]
                  },
                  Slider {
                      min: 1
                      max: 50
                      vertical: false
                      value: bind radius with inverse
                      translateX: 40
                      translateY: 140
                  }
              ]
           }
}
