package javafxanimation;

import javafx.animation.Timeline;
import javafx.scene.CustomNode;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.scene.Scene;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;

import javafx.scene.control.Button;

var node: Node;
var panel: Node;
var scene: Scene;
Stage {
    title: "Animation #8"
    resizable: false
    scene: scene = Scene {
            width: 500
            height: 280
            fill: Color.GREY
            content: [
                node = Wheel {
                    fill: Color.YELLOW
                    stroke: Color.BLACK
                    radius: 50
                }
                panel = ControlPanel {
                    translateY: bind node.layoutBounds.height
                    width: bind scene.width
                    height: bind scene.height - panel.translateY
                }
            ]
        }
}

var timeline = Timeline {
    autoReverse: true
    repeatCount: Timeline.INDEFINITE
    keyFrames: [
        at (3s) {
        node.translateX =>
        scene.width - node.boundsInLocal.width;
        node.rotate => 360;
        }
    ]
}

var running = bind timeline.running;
var paused = bind timeline.paused;
var rate = bind timeline.rate;
var currentRate = bind timeline.currentRate;
var time = bind timeline.time;

class ControlPanel extends CustomNode {
    var width: Number;
    var height: Number;
    protected override function create():Node {
        var font = Font {
            name: "Serif"
            size: 20
        }
        Group {
            content: [
                Rectangle {
                    width: bind width
                    height: bind height
                    fill: Color.WHITE
                }
                Text {
                    x: 10
                    y: 20
                    font: font
                    content: bind "Rate: {rate}"
                }
                Text {
                    x: 170
                    y: 20
                    font: font
                    content: bind "Current Rate: {currentRate}"
                }
                Text {
                    x: 340
                    y: 20
                    font: font
                    content: bind "Time: {time}"
                }
                Text {
                    x: 10
                    y: 50
                    font: font
                    content: bind "Running: {running}"
                }
                Text {
                    x: 170
                    y: 50
                    font: font
                    content: bind "Paused: {paused}"
                }
                Button {
                    translateX: 180
                    translateY: 80
                    text: bind if (running) "Stop" else "Start"
                    action: function() {
                        if (running) {
                            timeline.stop()
                        } else {
                            timeline.rate = 1.0;
                            timeline.playFromStart();
                        }
                    }
                }
                Button {
                    translateX: 270
                    translateY: 80
                    disable: bind not running
                    text: bind if (paused) "Resume" else "  Pause  "
                    action: function() {
                        if (timeline.paused) {
                            timeline.play()
                        } else {
                            timeline.pause();
                        }
                    }
                }
                Button {
                    translateX: 90
                    translateY: 120
                    disable: bind not running
                    text: "Increase Rate"
                    action: function() {
                        if (timeline.rate >= 0) {
                            timeline.rate++;
                        } else {
                            timeline.rate;
                        }
                    }
                }
                Button {
                    translateX: 210
                    translateY: 120
                    disable: bind not running
                    text: "Decrease Rate"
                    action: function() {
                        if (timeline.rate >= 0) {
                            timeline.rate--;
                        } else {
                            timeline.rate++;
                        }
                    }
                }
                Button {
                    translateX: 330
                    translateY: 120
                    disable: bind not running
                    text: "Toggle Rate"
                    action: function() {
                        timeline.rate = -timeline.rate;
                    }
                }
            ]
        }
    }
}
