package javafxcharts;

import javafx.scene.Scene;
import javafx.scene.chart.BarChart;
import javafx.stage.Stage;
import javafx.scene.chart.part.CategoryAxis;
import javafx.scene.chart.part.NumberAxis;
import javafx.scene.chart.part.Side;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontPosture;
import javafx.scene.text.FontWeight;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;
import javafx.geometry.Insets;

var categories = [ "Duration", "EVA Time", "Samples (kg)" ];
var seriesData = [
    BarChart.Series {
        name: "Apollo 11"
        data: [
            BarChart.Data { category: "Duration" value: 195.5 }
            BarChart.Data { category: "EVA Time" value: 2.5 }
            BarChart.Data { category: "Samples (kg)" value: 22 }
        ]
    }
    BarChart.Series {
        name: "Apollo 15"
        data: [
            BarChart.Data { category: "Duration" value: 295.2 }
            BarChart.Data { category: "EVA Time" value: 19.2 }
            BarChart.Data { category: "Samples (kg)" value: 76.8 }
        ]
    }
    BarChart.Series {
        name: "Apollo 16"
        data: [
            BarChart.Data { category: "Duration" value: 265.9 }
            BarChart.Data { category: "EVA Time" value: 20.3 }
            BarChart.Data { category: "Samples (kg)" value: 94.7 }
        ]
    }
    BarChart.Series {
        name: "Apollo 17"
        data: [
            BarChart.Data { category: "Duration" value: 301.8 }
            BarChart.Data { category: "EVA Time" value: 22.1 }
            BarChart.Data { category: "Samples (kg)" value: 110.5 }
        ]
    }
];

var chart:BarChart;
Stage {
    title: "Customized Chart #1"
    scene: Scene {
        content: [
            chart = BarChart {
                title: "Apollo Lunar Landings"
                titleSide: Side.BOTTOM
                titleFill: Color.GREEN
                titleFont: Font.font("Serif", FontWeight.REGULAR,
                                      FontPosture.ITALIC, 16);
                chartBackgroundFill: LinearGradient {
                    startX: 0 endX: 0 startY: 0 endY: 1
                    stops: [
                        Stop { offset: 0 color: Color.GREY }
                        Stop { offset: 1 color: Color.LIGHTGRAY }
                    ]
                }
                chartBackgroundStroke: Color.BLACK
                chartBackgroundStrokeWidth: 4
                insets: Insets { bottom: 32 left: 8
                                 top: 32 right: 8}
                legendSide: Side.LEFT
                data: seriesData
                categoryAxis: CategoryAxis {
                    categories: categories
                }
                valueAxis: NumberAxis {
                    upperBound: 310
                    tickUnit: 100
                }
            }
        ]
    }
}

chart.legend.fill = Color.YELLOW;
