package javafxservercode;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class HttpRequestServlet extends HttpServlet {
    /**
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
                         throws ServletException, IOException {
        PrintWriter out = response.getWriter();
        try {
            response.setContentType("text/plain");
            response.setStatus(HttpServletResponse.SC_OK);
            out.println("HTTPRequestServlet");
        } finally {
            out.close();
        }
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response)
                          throws ServletException, IOException {
        StringBuffer sb = new StringBuffer();
        Enumeration names = request.getParameterNames();
        while (names.hasMoreElements()) {
            String name = (String)names.nextElement();
            sb.append(name);
            sb.append("=");
            sb.append(request.getParameter(name));
            sb.append("|");
        }
        response.sendError(HttpServletResponse.SC_OK, sb.toString());
    }

    /**
     * Handles the HTTP <code>PUT</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPut(HttpServletRequest request,
                         HttpServletResponse response)
                         throws ServletException, IOException {
        int count = 0;
        InputStream is = request.getInputStream();
        try {
            while (is.read() != -1) {
                count++;
            }
        } finally {
            is.close();
        }
        String type = request.getContentType();
        response.sendError(HttpServletResponse.SC_OK,
                           "Got " + count + " bytes, type " + type);
    }
    
    /**
     * Handles the HTTP <code>DELETE</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doDelete(HttpServletRequest request,
                            HttpServletResponse response)
                            throws ServletException, IOException {
        String path = request.getPathInfo();
        if (path != null) {
            response.sendError(HttpServletResponse.SC_OK,
                               "Deleted file " + path);
        } else {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
        }
    }

    /**
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "HttpRequest Servlet";
    }
}
